

#property indicator_chart_window
#property indicator_buffers 3
#property indicator_color1  Green
#property indicator_color2  Red
#property indicator_color3  Red
#property indicator_width1  2
#property indicator_width2  2
#property indicator_width3  2

extern string TimeFrame       = "Current time frame";
extern int    MAPeriod        = 14;
extern int    MAType          = 1;
extern int    MaShift         = 0;
extern int    Applied_Price   = PRICE_CLOSE;
extern bool   alertsOn        = true;
extern bool   alertsOnCurrent = false;
extern bool   alertsMessage   = true;
extern bool   alertsNotify    = false;
extern bool   alertsSound     = false;
extern bool   alertsEmail     = false;


//---- buffers

double ExtMapBuffer1[];
double ExtMapBuffer2[];
double ExtMapBuffer3[];
double slope[];
double trend[];

//---- variables

int    timeFrame;
int    MAMode;
string strMAType;


//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
  {
   IndicatorBuffers(5);
   SetIndexBuffer(0,ExtMapBuffer3); SetIndexShift(0,MaShift);
   SetIndexBuffer(1,ExtMapBuffer2); SetIndexShift(1,MaShift);
   SetIndexBuffer(2,ExtMapBuffer1); SetIndexShift(2,MaShift);
   SetIndexBuffer(3,slope);
   SetIndexBuffer(4,trend);
   
   switch (MAType)
   {
      case 1:  strMAType = " EMA ";  MAMode = MODE_EMA;  break;
      case 2:  strMAType = " SMMA "; MAMode = MODE_SMMA; break;
      case 3:  strMAType = " LWMA "; MAMode = MODE_LWMA; break;
      case 4:  strMAType = " LSMA ";                     break;
      default: strMAType = " SMA ";  MAMode = MODE_SMA;  break;
   }
   timeFrame = stringToTimeFrame(TimeFrame);
   IndicatorShortName(timeFrameToString(timeFrame)+ strMAType+ " (" +MAPeriod + ") ");
   return(0);
  }


int start()
{
   int counted_bars=IndicatorCounted();
      if(counted_bars<0) return(-1);
      if(counted_bars>0) counted_bars--;
            int limit=MathMax(Bars-counted_bars,3*timeFrame/Period());
   
   //
   //
   //
   //
   //
   
   if (slope[limit]==-1) CleanPoint(limit,ExtMapBuffer2,ExtMapBuffer1);
   for(int i=limit; i>=0; i--)
   {
      int y = iBarShift(NULL,timeFrame,Time[i]);
      if (MAType == 4)
      {
        ExtMapBuffer3[i] = 3.0*iMA(NULL,timeFrame,MAPeriod,MaShift,MODE_LWMA,Applied_Price,y)-2.0*iMA(NULL,timeFrame,MAPeriod,MaShift,MODE_SMA,Applied_Price,y);
      }
      else
      {
        ExtMapBuffer3[i] = iMA(NULL,timeFrame,MAPeriod,MaShift,MAMode,Applied_Price,y);
      }
        ExtMapBuffer2[i] = EMPTY_VALUE;
        ExtMapBuffer1[i] = EMPTY_VALUE;
        slope[i] = slope[i+1];
        trend[i] = trend[i+1];
        if(ExtMapBuffer3[i]>ExtMapBuffer3[i+1]) slope[i] = 1;
        if(ExtMapBuffer3[i]<ExtMapBuffer3[i+1]) slope[i] =-1;
        if(Close[i]>ExtMapBuffer3[i])           trend[i] = 1;
        if(Close[i]<ExtMapBuffer3[i])           trend[i] =-1;
        if (slope[i]==-1) PlotPoint(i,ExtMapBuffer2,ExtMapBuffer1,ExtMapBuffer3);
   }
   
   //
   //
   //
   //
   //
   
   if (alertsOn)
   {
      if (alertsOnCurrent)
            int whichBar = 0;
      else      whichBar = 1; whichBar = iBarShift(NULL,0,iTime(NULL,timeFrame,whichBar));
      if (trend[whichBar] !=  trend[whichBar+1])
      if (trend[whichBar] == 1)
            doAlert("up");
      else  doAlert("down");       
   }   
return(0);
}

//+-------------------------------------------------------------------
//|                                                                  
//+-------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = stringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}

//
//
//
//
//

string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//
//
//
//
//

string stringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int tchar = StringGetChar(s, length);
         if((tchar > 96 && tchar < 123) || (tchar > 223 && tchar < 256))
                     s = StringSetChar(s, length, tchar - 32);
         else if(tchar > -33 && tchar < 0)
                     s = StringSetChar(s, length, tchar + 224);
   }
   return(s);
}

//-------------------------------------------------------------------
//                                                                  
//-------------------------------------------------------------------
//
//
//
//
//

void CleanPoint(int i,double& first[],double& second[])
{
   if ((second[i]  != EMPTY_VALUE) && (second[i+1] != EMPTY_VALUE))
        second[i+1] = EMPTY_VALUE;
   else
      if ((first[i] != EMPTY_VALUE) && (first[i+1] != EMPTY_VALUE) && (first[i+2] == EMPTY_VALUE))
          first[i+1] = EMPTY_VALUE;
}

//
//
//
//
//

void PlotPoint(int i,double& first[],double& second[],double& from[])
{
   if (first[i+1] == EMPTY_VALUE)
      {
         if (first[i+2] == EMPTY_VALUE) {
                first[i]   = from[i];
                first[i+1] = from[i+1];
                second[i]  = EMPTY_VALUE;
            }
         else {
                second[i]   =  from[i];
                second[i+1] =  from[i+1];
                first[i]    = EMPTY_VALUE;
            }
      }
   else
      {
         first[i]  = from[i];
         second[i] = EMPTY_VALUE;
      }
}




//------------------------------------------------------------------
//                                                                  
//------------------------------------------------------------------
//
//
//
//
//

void doAlert(string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
      if (previousAlert != doWhat || previousTime != Time[0]) {
          previousAlert  = doWhat;
          previousTime   = Time[0];

          //
          //
          //
          //
          //

          message =  StringConcatenate(Symbol()," ",timeFrameToString(timeFrame)," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," ","Price crossing", +MAPeriod+ " Period",strMAType+" nrp Color ",doWhat);
             if (alertsMessage) Alert(message);
             if (alertsNotify)  SendNotification(message);
             if (alertsEmail)   SendMail(StringConcatenate(Symbol(),"Price crossing",strMAType+" nrp Color "),message);
             if (alertsSound)   PlaySound("alert2.wav");
      }
}

